@@ -104,8 +104,13 @@ set_gerror_from_pcre_error(int errcode,
         return FALSE;
 }
 
+#pragma GCC diagnostic push
+#if defined(__GNUC__) && !defined(__clang__)
+#pragma GCC diagnostic ignored "-Wcast-function-type"
+#endif
 G_DEFINE_BOXED_TYPE(VteRegex, vte_regex,
                     vte_regex_ref, (GBoxedFreeFunc)vte_regex_unref)
+#pragma GCC diagnostic pop
 
 G_DEFINE_QUARK(vte-regex-error, vte_regex_error)
 
@@ -147,6 +152,36 @@ vte_regex_unref(VteRegex *regex)
         return NULL;
 }
 
+static gboolean
+check_pcre_config_unicode(GError** error)
+{
+        /* Check library compatibility */
+        guint32 v;
+        int r = pcre2_config_8(PCRE2_CONFIG_UNICODE, &v);
+        if (r != 0 || v != 1) {
+                g_set_error(error, VTE_REGEX_ERROR, VTE_REGEX_ERROR_INCOMPATIBLE,
+                            "PCRE2 library was built without unicode support");
+                return FALSE;
+        }
+
+        return TRUE;
+}
+
+static gboolean
+check_pcre_config_jit(void)
+{
+        static gboolean warned = FALSE;
+
+        char s[256];
+        int r = pcre2_config_8(PCRE2_CONFIG_JITTARGET, &s);
+        if (r == PCRE2_ERROR_BADOPTION && !warned) {
+                g_printerr("PCRE2 library was built without JIT support\n");
+                warned = TRUE;
+        }
+
+        return r >= 1;
+}
+
 static VteRegex *
 vte_regex_new(VteRegexPurpose purpose,
               const char *pattern,
@@ -155,21 +190,15 @@ vte_regex_new(VteRegexPurpose purpose,
               GError    **error)
 {
         pcre2_code_8 *code;
-        int r, errcode;
-        guint32 v;
+        int errcode;
         PCRE2_SIZE erroffset;
 
         g_return_val_if_fail(pattern != NULL, NULL);
         g_return_val_if_fail(pattern_length >= -1, NULL);
         g_return_val_if_fail(error == NULL || *error == NULL, NULL);
 
-        /* Check library compatibility */
-        r = pcre2_config_8(PCRE2_CONFIG_UNICODE, &v);
-        if (r != 0 || v != 1) {
-                g_set_error(error, VTE_REGEX_ERROR, VTE_REGEX_ERROR_INCOMPATIBLE,
-                            "PCRE2 library was built without unicode support");
-                return NULL;
-        }
+        if (!check_pcre_config_unicode(error))
+                return FALSE;
 
         code = pcre2_compile_8((PCRE2_SPTR8)pattern,
                                pattern_length >= 0 ? pattern_length : PCRE2_ZERO_TERMINATED,
@@ -213,7 +242,8 @@ _vte_regex_new_gregex(VteRegexPurpose purpose,
                 { G_REGEX_DUPNAMES,        PCRE2_DUPNAMES        }
         };
 
-        guint32 gflags = g_regex_get_compile_flags(gregex);
+        /* Always add the MULTILINE option */
+        guint32 gflags = g_regex_get_compile_flags(gregex) | G_REGEX_MULTILINE;
         translate_flags(table, G_N_ELEMENTS(table), &gflags, &pflags);
 
         if (gflags != 0) {
@@ -369,7 +399,8 @@ _vte_regex_get_pcre(VteRegex *regex)
  *
  * If the platform supports JITing, JIT compiles @regex.
  *
- * Returns: %TRUE if JITing succeeded, or %FALSE with @error filled in
+ * Returns: %TRUE if JITing succeeded (or PCRE2 was built without
+ *   JIT support), or %FALSE with @error filled in
  */
 gboolean
 vte_regex_jit(VteRegex *regex,
@@ -380,6 +411,9 @@ vte_regex_jit(VteRegex *regex,
 
         g_return_val_if_fail(regex != NULL, FALSE);
 
+        if (!check_pcre_config_jit())
+                return TRUE;
+
         r = pcre2_jit_compile_8(regex->code, flags);
         if (r < 0)
                 return set_gerror_from_pcre_error(r, error);
@@ -407,3 +441,19 @@ _vte_regex_get_jited(VteRegex *regex)
 
         return r == 0 && s != 0;
 }
+
+/*
+ * _vte_regex_get_compile_flags:
+ *
+ * Returns: the PCRE2 flags used to compile @regex
+ */
+guint32
+_vte_regex_get_compile_flags(VteRegex *regex)
+{
+        g_return_val_if_fail(regex != nullptr, 0);
+
+        uint32_t v;
+        int r = pcre2_pattern_info_8(regex->code, PCRE2_INFO_ARGOPTIONS, &v);
+
+        return r == 0 ? v : 0u;
+}
