@@ -36,10 +36,10 @@
 #include <sys/resource.h>
 #endif /* HAVE_SYS_RESOURCE_H */
 
-#include <glib/gstdio.h>
 #include <glib-unix.h>
 
 #include "vtespawn.hh"
+#include "vteutils.h"  /* for strchrnul on non-GNU systems */
 #include "reaper.hh"
 
 #define VTE_SPAWN_ERROR_TIMED_OUT (G_SPAWN_ERROR_FAILED + 1000)
@@ -154,7 +154,7 @@ close_and_invalidate (gint *fd)
     return;
   else
     {
-      (void) g_close (*fd, NULL);
+      (void)close(*fd);
       *fd = -1;
     }
 }
@@ -510,7 +510,7 @@ enum
   CHILD_FORK_FAILED
 };
 
-static void
+static G_GNUC_NORETURN void
 do_exec (gint                  child_err_report_fd,
          gint                  stdin_fd,
          gint                  stdout_fd,
@@ -1002,16 +1002,6 @@ script_execute (const gchar *file,
   }
 }
 
-static gchar*
-my_strchrnul (const gchar *str, gchar c)
-{
-  gchar *p = (gchar*) str;
-  while (*p && (*p != c))
-    ++p;
-
-  return p;
-}
-
 static gint
 g_execute (const gchar *file,
            gchar      **argv,
@@ -1082,7 +1072,7 @@ g_execute (const gchar *file,
 	  char *startp;
 
 	  path = p;
-	  p = my_strchrnul (path, ':');
+	  p = strchrnul (path, ':');
 
 	  if (p == path)
 	    /* Two adjacent colons, or a colon at the beginning or the end
